/*
 *     jsz.Plotter - a SWING-based Java package for plotting data
 * Copyright (C) 2000 Janek Schwarz <schwarz@informatik.hu-berlin.de>
 *
 *                    This software is dedicated to 
 *                             THE RAMONES
 *                       the best punk band ever.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: DataSeries.java,v 1.5 2000/02/03 21:01:23 schwarz Exp $
 */
package jsz.Plotter;

import java.awt.Color;
import java.util.*;

/**
 * Straight-foreward implemenation of {@link
 * jsz.Plotter.IDataSeries}. This implementation supports
 * multicolored graphs. A dataseries is either plotted as
 * one-colored (default) or multi-colored line. A one-colored
 * graph turns into a multi-colored one when adding DataSets with
 * different colors (see {@link
 * jsz.Plotter.DataSet#setColor(Color c)} and {@link
 * jsz.Plotter.DataSet#DataSet(double x, double y, Color color)}
 * and {@link #addDataSet(double x, double y, Color
 * c)}). DataSets which doesn't define their own color are still
 * plotted with the default color, even if their added after a
 * colored DataSet. In other words, DataSets succeeding a colored
 * DataSet don't inherit that DataSet's color.
 */
public class DataSeries
  extends Observable
  implements IDataSeries
{
/**
 * name this instance is associated with. It is usually used as a legend.
 */
private String legend = null;

/**
 * vector, where the DataSets are stored in
 */
private Vector data   = new Vector();

/**
 * default color the data is plotted with
 */
private Color  color  = Color.black;

/**
 * minimal x-value in data
 */
private double xmin   = 0;

/**
 * maximal x-value in data
 */
private double xmax   = 0;

/**
 * minimal y-value in data
 */
private double ymin   = 0;

/**
 * maximal y-value in data
 */
private double ymax   = 0;

/**
 * Describe constructor here.
 *
 * @param legend name associated with this dataseries (legend)
 * @param color default color the data is plotted with
 */
public DataSeries(String legend,
                  Color color)
{
  this.legend = legend;
  this.color  = color;
}

/**
 * See {@link jsz.Plotter.IDataSeries#addDataSet(DataSet ds)}.
 * Additionaly, if ds doesn't have a color associated, the default color
 * (see {@link #color}) is assigned to it.
 */
public IDataSeries addDataSet(DataSet ds)
{
  double x = ds.getX();
  double y = ds.getY();
  
  if(x < xmin)
    xmin = x;
  else if(x > xmax)
    xmax = x;

  if(y < ymin)
    ymin = y;
  else if(y > ymax)
    ymax = y;
  
  data.addElement(ds);
  this.setChanged();
  this.notifyObservers(new UpdateDescription(this, UpdateDescription.DATA));

  return this;
}

/**
 * See {@link jsz.Plotter.IDataSeries#addDataSet(double x,
 * double y)}. Additionaly, if ds doesn't have a color associated, the
 * default color (see {@link #color}) is assigned to it.
 */
public IDataSeries addDataSet(double x, double y)
{
  return this.addDataSet(new DataSet(x, y));
}

public IDataSeries addDataSet(double x, double y, Color c)
{
  return this.addDataSet(new DataSet(x, y, c));
}

public final void setColor(Color c)
{
  color = c;
  this.setChanged();
  this.notifyObservers(new UpdateDescription(this, 
                                             UpdateDescription.GRAPH_COLOR));
}

public final Color getColor()
{
  return color;
}

public final void setLegend(String name)
{
  legend = name;
  this.setChanged();
  this.notifyObservers(new UpdateDescription(this, UpdateDescription.LEGEND));
}

public final String getLegend()
{
  return legend;
}

public final Vector getData()
{
  return data;
}

public final double getXMin()
{
  return xmin;
}

public final double getXMax()
{
  return xmax;
}

public final double getYMin()
{
  return ymin;
}

public final double getYMax()
{
  return ymax;
}
};
